@@ -4,6 +4,7 @@ from urllib3.connectionpool import (
     connection_from_url,
     HTTPConnection,
     HTTPConnectionPool,
+    HTTPSConnectionPool,
 )
 from urllib3.util.timeout import Timeout
 from urllib3.packages.ssl_match_hostname import CertificateError
@@ -74,6 +75,54 @@ class TestConnectionPool(unittest.TestCase):
             c = connection_from_url(b)
             self.assertFalse(c.is_same_host(a), "%s =? %s" % (b, a))
 
+    def test_same_host_no_port(self):
+        # This test was introduced in #801 to deal with the fact that urllib3
+        # never initializes ConnectionPool objects with port=None.
+        same_host_http = [
+            ('google.com', '/'),
+            ('google.com', 'http://google.com/'),
+            ('google.com', 'http://google.com'),
+            ('google.com', 'http://google.com/abra/cadabra'),
+            # Test comparison using default ports
+            ('google.com', 'http://google.com:80/abracadabra'),
+        ]
+        same_host_https = [
+            ('google.com', '/'),
+            ('google.com', 'https://google.com/'),
+            ('google.com', 'https://google.com'),
+            ('google.com', 'https://google.com/abra/cadabra'),
+            # Test comparison using default ports
+            ('google.com', 'https://google.com:443/abracadabra'),
+        ]
+
+        for a, b in same_host_http:
+            c = HTTPConnectionPool(a)
+            self.assertTrue(c.is_same_host(b), "%s =? %s" % (a, b))
+        for a, b in same_host_https:
+            c = HTTPSConnectionPool(a)
+            self.assertTrue(c.is_same_host(b), "%s =? %s" % (a, b))
+
+        not_same_host_http = [
+            ('google.com', 'https://google.com/'),
+            ('yahoo.com', 'http://google.com/'),
+            ('google.com', 'https://google.net/'),
+        ]
+        not_same_host_https = [
+            ('google.com', 'http://google.com/'),
+            ('yahoo.com', 'https://google.com/'),
+            ('google.com', 'https://google.net/'),
+        ]
+
+        for a, b in not_same_host_http:
+            c = HTTPConnectionPool(a)
+            self.assertFalse(c.is_same_host(b), "%s =? %s" % (a, b))
+            c = HTTPConnectionPool(b)
+            self.assertFalse(c.is_same_host(a), "%s =? %s" % (b, a))
+        for a, b in not_same_host_https:
+            c = HTTPSConnectionPool(a)
+            self.assertFalse(c.is_same_host(b), "%s =? %s" % (a, b))
+            c = HTTPSConnectionPool(b)
+            self.assertFalse(c.is_same_host(a), "%s =? %s" % (b, a))
 
     def test_max_connections(self):
         pool = HTTPConnectionPool(host='localhost', maxsize=1, block=True)
@@ -205,6 +254,61 @@ class TestConnectionPool(unittest.TestCase):
     def test_no_host(self):
         self.assertRaises(LocationValueError, HTTPConnectionPool, None)
 
+    def test_contextmanager(self):
+        with connection_from_url('http://google.com:80') as pool:
+            # Populate with some connections
+            conn1 = pool._get_conn()
+            conn2 = pool._get_conn()
+            conn3 = pool._get_conn()
+            pool._put_conn(conn1)
+            pool._put_conn(conn2)
+
+            old_pool_queue = pool.pool
+
+        self.assertEqual(pool.pool, None)
+        self.assertRaises(ClosedPoolError, pool._get_conn)
+
+        pool._put_conn(conn3)
+        self.assertRaises(ClosedPoolError, pool._get_conn)
+        self.assertRaises(Empty, old_pool_queue.get, block=False)
+
+    def test_absolute_url(self):
+        c = connection_from_url('http://google.com:80')
+        self.assertEqual(
+                'http://google.com:80/path?query=foo',
+                c._absolute_url('path?query=foo'))
+
+    def test_ca_certs_default_cert_required(self):
+        with connection_from_url('https://google.com:80', ca_certs='/etc/ssl/certs/custom.pem') as pool:
+            conn = pool._get_conn()
+            self.assertEqual(conn.cert_reqs, 'CERT_REQUIRED')
+
+    def test_cleanup_on_extreme_connection_error(self):
+        """
+        This test validates that we clean up properly even on exceptions that
+        we'd not otherwise catch, i.e. those that inherit from BaseException
+        like KeyboardInterrupt or gevent.Timeout. See #805 for more details.
+        """
+        class RealBad(BaseException):
+            pass
+
+        def kaboom(*args, **kwargs):
+            raise RealBad()
+
+        c = connection_from_url('http://localhost:80')
+        c._make_request = kaboom
+
+        initial_pool_size = c.pool.qsize()
+
+        try:
+            # We need to release_conn this way or we'd put it away regardless.
+            c.urlopen('GET', '/', release_conn=False)
+        except RealBad:
+            pass
+
+        new_pool_size = c.pool.qsize()
+        self.assertEqual(initial_pool_size, new_pool_size)
+
 
 
 if __name__ == '__main__':
